#include "networkhandler.h"

NetworkHandler::NetworkHandler(MainWindow *parent)
{
    myParent = parent;
    isTest = false;
    storedMaxPlayers = 0;
    storedOnlinePlayers = 0;
    packetSeparator = "";
    packetSeparator.append(QChar(25)).append(QChar(26));
    connect(myParent->socket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(erreurSocket(QAbstractSocket::SocketError)));
    connect(myParent->socket, SIGNAL(connected()), this, SLOT(connected()));
    connect(myParent->socket, SIGNAL(disconnected()), this, SLOT(disconnected()));
    connect(myParent->socket, SIGNAL(readyRead()), this, SLOT(datasReceived()));
}

void NetworkHandler::setTestServer(const Server &server)
{
    testServer = server;
    isTest = true;
    myParent->socket->connectToHost(server.serverAddress, server.serverPort.toInt());
}

void NetworkHandler::endTestServer()
{
    Server server;
    testServer = server;
    isTest = false;
    myParent->socket->disconnectFromHost();
}

void NetworkHandler::sendSocket(QString message)
{
    qDebug() << ">" << message;
    message.append("\r\n");
    myParent->socket->write(qPrintable(message));
    myParent->socket->flush();
}

void NetworkHandler::disconnectFromHost()
{
    myParent->socketConnected = false;
    myParent->socket->disconnectFromHost();
    myParent->credentialsSent = false;
}

void NetworkHandler::erreurSocket(QAbstractSocket::SocketError erreur)
{
    QString message;
    switch (erreur)
    {
    case QAbstractSocket::HostNotFoundError:
        message = tr("The server wasn't found. Verify IP and port.");
        break;
    case QAbstractSocket::ConnectionRefusedError:
        message = tr("The server refused the connection. Verify if the server is launched.");
        break;
    case QAbstractSocket::RemoteHostClosedError:
        message = tr("The server closed the connection.");
        break;
    default:
        message = myParent->socket->errorString();
    }
    QMessageBox qmbox(QMessageBox::Warning, tr("Error connecting to server"), message, QMessageBox::Ok, myParent, Qt::Dialog);
    qmbox.setIconPixmap(QPixmap(":/images/icones/server_error"));
    qmbox.exec();

}
void NetworkHandler::connected()
{
    if(isTest)
    {
        sendSocket("HELLO test");
    }
    else
    {
        sendSocket("HELLO");
    }
}

void NetworkHandler::disconnected()
{
    myParent->socketConnected = false;
}
void NetworkHandler::datasReceived()
{
    QTcpSocket *socket = qobject_cast<QTcpSocket *> (sender());
    if (socket == 0)
    {
        qDebug() << "Sender not found";
        return;
    }

    QString paquet;
    paquet = QString::fromUtf8(myParent->socket->readAll());
    qDebug() << "< " << paquet;

    QStringList paquets = paquet.split(packetSeparator);
    for (int i=0;i<paquets.size();i++)
    {
        if(paquets.at(i).length()>0)
        {
            processPaquet(paquets.at(i));
        }
    }
}

void NetworkHandler::processPaquet(QString paquet)
{
    if (!myParent->credentialsSent)
    {
        if (!myParent->socketConnected && paquet.startsWith("HELLO"))
        {
            double serverVersion = paquet.replace("HELLO ", "").toDouble();
            if(serverVersion != myParent->version)
            {
                emit versionMismatch((serverVersion > (myParent->version)));
                myParent->socket->disconnectFromHost();
                return;
            }
            myParent->socketConnected = true;
            QString userPacket = "USER ";
            if(this->isTest)
            {
                currentUsername = this->testServer.username;
            }
            else
            {
                currentUsername = myParent->logins.at(myParent->serverID.toInt()).username;
            }
            userPacket.append(currentUsername);
            sendSocket(userPacket);
        }
        else if (myParent->socketConnected && paquet.startsWith("100 Please send password"))
        {
            QString passPacket = "PASS ";
            if(this->isTest)
            {
                passPacket.append(this->testServer.password);
            }
            else
            {
                passPacket.append(myParent->logins.at(myParent->serverID.toInt()).password);
            }
            sendSocket(passPacket);
            myParent->credentialsSent = true;
        }
    }
    else
    {
        if (paquet == "200 Logged in")
        {
            emit ssocketConnected();
            if(this->isTest)
            {
                disconnectFromHost();
            }
            else
            {
                sendSocket("BASICINFOS");
                sendSocket("SERVERINFOS");
            }
        }
        else if (paquet == "403 Username or password incorrect")
        {
            disconnectFromHost();
            emit socketCredentialsIncorrect();
        }
        else if(paquet.startsWith("RIGHTS"))
        {
            processRights(paquet);
        }
        else if(paquet.startsWith("BASICINFOS"))
        {
            processBasicInfos(paquet);
        }
        else if(paquet.startsWith("SERVERINFOS"))
        {
            processServerInfos(paquet);
        }
        else if(paquet.startsWith("CHATMSG"))
        {
            processChatMsg(paquet);
        }
        else if(paquet.startsWith("PLAYERJOIN"))
        {
            processJoin(paquet);
        }
        else if(paquet.startsWith("PLAYERQUITS"))
        {
            processQuit(paquet);
        }
    }
}

void NetworkHandler::processRights(QString paquet)
{
    if(!paquet.startsWith("RIGHTS"))
        return;

    paquet.replace("RIGHTS ", "");
    QStringList droits = paquet.split("|");
    if(droits.length()!=1)
        return;

    if(droits.at(0) == "0")
    {
        myParent->getUi()->reloadButton->setEnabled(false);
        myParent->getUi()->stopButton->setEnabled(false);
    }
}

void NetworkHandler::processBasicInfos(QString paquet)
{
    if(!paquet.startsWith("BASICINFOS"))
        return;

    paquet.replace("BASICINFOS ", "");
    QStringList infos = paquet.split("|");
    if(infos.length()!=4)
        return;

    myParent->getUi()->OSText->setText(infos.at(0)+" "+infos.at(1));
    if(infos.at(2).contains("Debian"))
    {
        myParent->getUi()->OsImg->setPixmap(QPixmap(":/images/os/debian.png"));
    }
    if(infos.at(0).contains("Windows"))
    {
        myParent->getUi()->OsImg->setPixmap(QPixmap(":/images/os/windows.png"));
    }
    myParent->getUi()->IPText->setText(infos.at(2));
    myParent->getUi()->UptimeText->setText(infos.at(3));
}

void NetworkHandler::processServerInfos(QString paquet)
{
    if(!paquet.startsWith("SERVERINFOS"))
        return;

    paquet.replace("SERVERINFOS ", "");
    QStringList infos = paquet.split("|");
    if(infos.length()!=7)
        return;

    currentServerLogin = myParent->socket->peerName()+":"+infos.at(0);

    myParent->getUi()->adresse->setText(myParent->getUi()->IPText->text()+":"+infos.at(0));
    myParent->getUi()->version->setText(infos.at(1));
    processUserCount(infos.at(2).toInt(), infos.at(3).toInt());
    myParent->getUi()->joueurs_whitelist->setText(infos.at(4));
    myParent->getUi()->plugins->setText(infos.at(5));
    if(infos.at(6).toInt() < 12000)
    {
        myParent->getUi()->heure->setText(tr("Day"));
    }
    else
    {
        myParent->getUi()->heure->setText(tr("Night"));
    }
}

void NetworkHandler::processUserCount(int onlinePlayers, int maxPlayers)
{
    storedMaxPlayers = maxPlayers;
    storedOnlinePlayers = onlinePlayers;
    QString color;
    if(onlinePlayers < (maxPlayers/3))
    {
        color = "gray";
    }
    else if(onlinePlayers < (maxPlayers/3*2))
    {
        color = "orange";
    }
    else if(onlinePlayers == maxPlayers)
    {
        color = "black";
    }
    else
    {
        color = "red";
    }
    myParent->getUi()->joueurs->setText("<span style=\"color:"+color+"\">"+QString::number(onlinePlayers)+"</span> / "+QString::number(maxPlayers));
}
void NetworkHandler::processUserCount(int onlinePlayers)
{
    processUserCount(storedOnlinePlayers+onlinePlayers, storedMaxPlayers);
}

void NetworkHandler::processChatMsg(QString paquet)
{
    if(!paquet.startsWith("CHATMSG"))
        return;

    paquet.replace("CHATMSG ", "");
    QStringList infos = paquet.split("|");
    if(infos.length()!=3)
        return;

    QString location;
    if(infos.at(1) == "ingame")
    {
        location = tr("In game", "Used in chat to tell where the user chat from");
    }
    else if(infos.at(1) == "gui")
    {
        location = tr("GUI", "Used in chat to tell where the user chat from");
    }
    myParent->getUi()->chatWidget->appendHtml("["+QTime::currentTime().toString("hh:mm:ss")+"] <strong>&lt;"+infos.at(0)+"&gt;</strong> (<span style=\"color:grey;\">"+location+"</span>) : "+infos.at(2));
}
void NetworkHandler::processJoin(QString paquet)
{
    if(!paquet.startsWith("PLAYERJOIN"))
        return;

    paquet.replace("PLAYERJOIN ", "");
    QStringList infos = paquet.split("|");
    if(infos.length()!=2)
        return;

    QString location, miniLoc;
    if(infos.at(1) == "ingame")
    {
        location = tr("connected from game", "Used in chat when an user connects from game");
        miniLoc = tr("IG", "Mini location for chat");
        processUserCount(+1);
    }
    else if(infos.at(1) == "gui")
    {
        location = tr("connected from GUI", "Used in chat when an user connects from GUI");
        miniLoc = tr("GUI", "Mini location for chat");
    }
    myParent->getUi()->chatWidget->appendHtml("<span style=\"color:grey;font-style:italic;\">"+infos.at(0)+" "+location+".</span>");
    myParent->getUi()->onlineList->addItem(new QListWidgetItem(infos.at(0)+" ("+miniLoc+")"));
}

void NetworkHandler::processQuit(QString paquet)
{
    if(!paquet.startsWith("PLAYERQUITS"))
        return;

    paquet.replace("PLAYERQUITS ", "");
    QStringList infos = paquet.split("|");
    if(infos.length()!=2)
        return;

    QString location, miniLoc;
    if(infos.at(1) == "ingame")
    {
        location = tr("left the game", "Used in chat when a player leaves the game");
        miniLoc = tr("IG", "Mini location for chat");
        processUserCount(-1);
    }
    else if(infos.at(1) == "gui")
    {
        location = tr("left GUI", "Used in chat when an user leaves the GUI");
        miniLoc = tr("GUI", "Mini location for chat");
    }
    myParent->getUi()->chatWidget->appendHtml("<span style=\"color:grey;font-style:italic;\">"+infos.at(0)+" "+location+".</span>");
    QListWidget* onlineList = myParent->getUi()->onlineList;
    delete onlineList->findItems(infos.at(0)+" ("+miniLoc+")", Qt::MatchExactly).at(0);
}

void NetworkHandler::sendChat(QString message)
{
    QString paquet = "CHAT ";
    paquet.append(message);
    sendSocket(paquet);
}
